local addon = MissionMinder;
addon.listview = {};
local listview = addon.listview;
local options = addon.options;
local DEBUG = MissionMinder.debug;

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");

MissionMinder.completeMissionMap = {};
function MissionMinder.IsMissionComplete(mission)
	return mission and MissionMinder.completeMissionMap[mission.missionID] ~= nil;
end


local function sortMissionsComparator(mission1, mission2)
	if ( options.ListViewEnabled() ) then
		local mission1Complete = MissionMinder.IsMissionComplete(mission1);
		local mission2Complete = MissionMinder.IsMissionComplete(mission2);
		if ( mission1Complete ~= mission2Complete ) then
			return mission1Complete;
		end
	end

	-- Blizzard code below --
	if ( mission1.level ~= mission2.level ) then
		return mission1.level > mission2.level;
	end

	if ( mission1.level == GARRISON_FOLLOWER_MAX_LEVEL ) then	-- mission 2 level is same as 1's at this point
		if ( mission1.iLevel ~= mission2.iLevel ) then
			return mission1.iLevel > mission2.iLevel;
		end
	end

	if ( mission1.durationSeconds ~= mission2.durationSeconds ) then
		return mission1.durationSeconds < mission2.durationSeconds;
	end

	if ( mission1.isRare ~= mission2.isRare ) then
		return mission1.isRare;
	end

	return strcmputf8i(mission1.name, mission2.name) < 0;
end


-- Alter sort to have completed missions at the top
local function sortMissionsCompletedFirst(missionsList)
	table.sort(missionsList, sortMissionsComparator);
end


-- Create tooltip for in progress misisons
local function createTooltipForCompletedMissions(...)
	local self, button = ...
	local mission = self.info;
	if (options.ListViewEnabled() and mission.inProgress) then
		GameTooltip:Hide();
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR_RIGHT");

		GameTooltip:SetText(mission.name);

		if ( MissionMinder.IsMissionComplete(mission) ) then
			GameTooltip:AddLine(L["(Complete)"], 0, 1, 0);
		else
			GameTooltip:AddLine(self.info.timeLeft.." "..RED_FONT_COLOR_CODE..GARRISON_MISSION_IN_PROGRESS..FONT_COLOR_CODE_CLOSE, 1, 1, 1);
		end

		--@debug@
		GameTooltip:AddLine("MissionID = " .. mission.missionID);
		--@end-debug

		GameTooltip:AddLine(" ");
		if mission.followers ~= nil then
			GameTooltip:AddLine(GARRISON_FOLLOWERS);
			for i=1, #(mission.followers) do
				GameTooltip:AddLine(C_Garrison.GetFollowerName(mission.followers[i]), 1, 1, 1);
			end
		end

		GameTooltip:Show();
	end
end

local function showInProgressMissionList(...)
	if ( options.ListViewEnabled() ) then
		PlaySound("UI_Garrison_CommandTable_ViewMissionReport");
		GarrisonMissionList_SetTab(GarrisonMissionFrame.MissionTab.MissionList.Tab2);
		GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog:Hide();
	else
		addon.GarrisonMissionFrame_ShowCompleteMissions(...);
	end
end

local function GetInProgressAndCompletedMissions(missions)
	C_Garrison.GetInProgressMissions(missions);

	if ( options.ListViewEnabled() ) then
		if ( not GarrisonMissionFrame.MissionComplete.completeMissions ) then
			GarrisonMissionFrame.MissionComplete.completeMissions = C_Garrison.GetCompleteMissions();
		end
		local completed = GarrisonMissionFrame.MissionComplete.completeMissions;

		-- Add completed mission to list
		for index = 1, #completed do
			local mission = completed[index];
			if ( mission.state == 0 ) then -- COMPLETED
				mission.inProgress = true;
				mission.timeLeft = ""; -- set to non-nil to avoid nil error in blizzard code
				missions[#missions + 1] = mission;
			end
		end
	end

	Garrison_SortMissions(missions);
end


-- Pretty much the same code as the standard UI, except that this calls GetInProgressAndCompletedMissions
local function missionListUpdate_IncludeCompletedMissions(...)
	local self = GarrisonMissionFrame.MissionTab.MissionList;
	GetInProgressAndCompletedMissions(self.inProgressMissions);
	C_Garrison.GetAvailableMissions(self.availableMissions);
	Garrison_SortMissions(self.availableMissions);
	self.Tab1:SetText(AVAILABLE.." - "..#self.availableMissions)
	self.Tab2:SetText(WINTERGRASP_IN_PROGRESS.." - "..#self.inProgressMissions)
	if ( #self.inProgressMissions > 0 ) then
		self.Tab2.Left:SetDesaturated(false);
		self.Tab2.Right:SetDesaturated(false);
		self.Tab2.Middle:SetDesaturated(false);
		self.Tab2.Text:SetTextColor(1, 1, 1);
		self.Tab2:SetEnabled(true);
	else
		self.Tab2.Left:SetDesaturated(true);
		self.Tab2.Right:SetDesaturated(true);
		self.Tab2.Middle:SetDesaturated(true);
		self.Tab2.Text:SetTextColor(0.5, 0.5, 0.5);
		self.Tab2:SetEnabled(false);
	end
	GarrisonMissionList_Update();
end

-- Pretty much the same code as the standard UI, except that this calls GetInProgressAndCompletedMissions
local function missionFrameUpdate_IncludeCompletedMissions(...)
	local self = GarrisonMissionFrame.MissionTab.MissionList;
	if (self.showInProgress) then
		GetInProgressAndCompletedMissions(self.inProgressMissions);
		self.Tab2:SetText(WINTERGRASP_IN_PROGRESS.." - "..#self.inProgressMissions)

		if( #self.inProgressMissions == 0) then
			self:SetScript("OnUpdate", nil);
		end
	else
		self.availableMissions = C_Garrison.GetAvailableMissions();
		Garrison_SortMissions(self.availableMissions);
		self.Tab1:SetText(AVAILABLE.." - "..#self.availableMissions)
		self:SetScript("OnUpdate", nil);
	end
	GarrisonMissionList_Update();
end



-- When user clicks on a mission from the mission list (either available or in progress)
local function viewMission(...)
	local self, button = ...

	if ( not options.ListViewEnabled() ) then
		return;
	end

	-- Ignore CHATLINK attempts and non-inProgress missions (handled properly already)
	if ( IsModifiedClick("CHATLINK") or not self.info.inProgress ) then
		return;
	end

	local mission = self.info;
	if ( MissionMinder.IsMissionComplete(mission) ) then -- Mission must be completed
		if ( options.QuickLoot() ) then
			if ( mission.state >= 0 ) then
				if ( C_Garrison.CanOpenMissionChest(mission.missionID) ) then
					C_Garrison.MissionBonusRoll(mission.missionID);
				else
					-- Mission cannot be looted; take no action
				end
			else
				C_Garrison.MarkMissionComplete(mission.missionID);
			end
		else
			-- This would be part of the OnClick event of the In Progress tab
			GarrisonMissionFrame.FollowerTab:Hide();
			GarrisonMissionFrame.FollowerList:Hide();
			HelpPlate_Hide();

			GarrisonMissionFrame.MissionComplete:Show();
			GarrisonMissionFrame.MissionCompleteBackground:Show();

			local self = GarrisonMissionFrame.MissionComplete;
			self.currentIndex = MissionMinder.completeMissionMap[mission.missionID].index;

			MissionCompletePreload_Cancel(); -- Cancel any previous preload that is still running
			MissionCompletePreload_LoadMission(mission.missionID); -- Start preload
			GarrisonMissionComplete_Initialize(self.completeMissions, self.currentIndex);
			return;
		end
	end
end



local function highlightCompletedMissions(...)
	-- Hide overlay on completed missions and highlight them as finished
	local self = GarrisonMissionFrame.MissionTab.MissionList;

	local missions = (self.showInProgress) and self.inProgressMissions or self.availableMissions;
	local numMissions = #missions;
	local scrollFrame = self.listScroll;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons = scrollFrame.buttons;
	local numButtons = #buttons;

	for i = 1, numButtons do
		local button = buttons[i];
		local index = offset + i; -- adjust index

		if ( index <= numMissions) then
			local mission = missions[index];

			if ( self.showInProgress ) then
				local _, _, _, successChance = C_Garrison.GetPartyMissionInfo(mission.missionID);
				button.SuccessChance:SetFormattedText(PERCENTAGE_STRING .. " " .. GARRISON_MISSION_CHANCE, successChance);
				button.SuccessChance:Show();
			else
				button.SuccessChance:SetText(nil);
				button.SuccessChance:Hide();
			end

			if( options.ListViewEnabled() and self.showInProgress and MissionMinder.IsMissionComplete(mission)) then
				button.Overlay:Hide();

				button.LocBG:SetVertexColor(0.3921, 1, 1, 1); -- green blue
				button.Summary:SetText(L["(Complete)"]);
				button.Summary:SetTextColor(0, 1, 0);
			else
				button.Summary:SetTextColor(1, 1, 1);
				button.LocBG:SetVertexColor(1, 1, 1, 1);
			end
		end
	end
end

local function createCompleteMissionMap(completeMissions)
	local map = {};
	for i = 1, #completeMissions do
		local mission = completeMissions[i];
		map[mission.missionID] = {};
		map[mission.missionID].index = i;
		map[mission.missionID].info = mission;
	end
	MissionMinder.completeMissionMap = map;
end

local function showMissionCompleteFrame(...)
	-- Do not popup mission complete dialog if the "In Progress" mission tab is already selected
	local self = GarrisonMissionFrame;
	if ( options.ListViewEnabled() ) then
		if( self:IsShown() and self.MissionTab.MissionList.showInProgress) then
			self.MissionComplete.completeMissions = C_Garrison.GetCompleteMissions();
			createCompleteMissionMap(self.MissionComplete.completeMissions);
			return;
		end
	end

	addon.GarrisonMissionFrame_CheckCompleteMissions(...)
	createCompleteMissionMap(self.MissionComplete.completeMissions)
end


local function closeMissionCompleteFrame(...)
	if ( options.ListViewEnabled() ) then
		GarrisonMissionFrame_HideCompleteMissions(true);
	else
		addon.GarrisonMissionCompleteNextButton_OnClick(...)
	end
end


local originalNextButtonText = nil;
local function updateNextMissionButtonText(...)
	local NextMissionButton = GarrisonMissionFrame.MissionComplete.NextMissionButton;

	if ( originalNextButtonText == nil ) then
		originalNextButtonText = NextMissionButton:GetText();
	end

	if ( options.ListViewEnabled() ) then
		NextMissionButton:SetText(L["Close"]);
	else
		NextMissionButton:SetText(originalNextButtonText);
	end
end


local function selectAvailableMissions(...)
	GarrisonMissionList_SetTab(GarrisonMissionFrame.MissionTab.MissionList.Tab1);
end

local function refreshCompletedMissions()
	if ( options.ListViewEnabled() ) then
		local self = GarrisonMissionFrame.MissionComplete;
		self.completeMissions = C_Garrison.GetCompleteMissions();
		createCompleteMissionMap(self.completeMissions);
	end
end

function listview.initialize()
	-- "View Completed Missions" button will open "In Progress" tab instead of "CompleteMission" frame
	addon.GarrisonMissionFrame_ShowCompleteMissions = GarrisonMissionFrame_ShowCompleteMissions;
	local ViewButton = GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog.BorderFrame.ViewButton;
	ViewButton:SetScript("OnClick", showInProgressMissionList);

	-- Completed Missions that haven't been looted are not considered "in progress" anymore
	--and need to be added to the list
	-- INSECURE HOOK - Needed to hook
	GarrisonMissionList_UpdateMissions = missionListUpdate_IncludeCompletedMissions;
	GarrisonMissionFrame_OnUpdate = missionFrameUpdate_IncludeCompletedMissions;

	-- Sort In Progress missions by completed first
	-- INSECURE HOOK - Seeing as the list to be sored is passed as an argument, this could be done securely.
	--However that would mean that we actually sort the list twice. I'm not sure it's worth it.
	Garrison_SortMissions = sortMissionsCompletedFirst;

	-- Allow completed "In Progress" missions to be interactable
	hooksecurefunc("GarrisonMissionList_Update", highlightCompletedMissions);

	-- Alter in progress, but completed mission tooltips
	-- OnClick "in progress" missions
	local buttons = GarrisonMissionFrame.MissionTab.MissionList.listScroll.buttons;
	local numButtons = #buttons;
	for i = 1, numButtons do
		local button = buttons[i];
		button:HookScript("OnClick", viewMission);
		button:HookScript("OnEnter", createTooltipForCompletedMissions);

		local successChance = button:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
		successChance:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, -2);
		successChance:SetTextColor(0, 1, 0);
		button.SuccessChance = successChance;
	end

	-- Mission finished event should not show complete dialog if "In Progress" tab already selected
	-- Currently uses a pre-hook, with more work might be able to remove the pre-hook requirement and use secure post-hook
	addon.GarrisonMissionFrame_CheckCompleteMissions = GarrisonMissionFrame_CheckCompleteMissions;
	GarrisonMissionFrame_CheckCompleteMissions = showMissionCompleteFrame;

	-- Closing GarrisonMissionFrame should select the "Available Missions" tab. This will cause the next open to show
	--the completed missions dialog again (if applicable)
	GarrisonMissionFrame:HookScript("OnHide", selectAvailableMissions);

	-- rewrite "Next" mission button as "Close"
	addon.GarrisonMissionCompleteNextButton_OnClick = GarrisonMissionCompleteNextButton_OnClick
	local NextMissionButton = GarrisonMissionFrame.MissionComplete.NextMissionButton;
	NextMissionButton:SetScript("OnClick", closeMissionCompleteFrame);
	local MissionComplete = GarrisonMissionFrame.MissionComplete;
	MissionComplete:HookScript("OnShow", updateNextMissionButtonText);

	-- Refresh completed missions list on GarrisonMissionFrame.MissionComplete:Hide
	GarrisonMissionFrame.MissionComplete:HookScript("OnHide", refreshCompletedMissions);
end